/**
 * Analytics event types
 * @public
 */
export declare enum FoundationAnalyticsEventType {
    controlClicked = "controlClicked",
    routeChanged = "routeChanged"
}
/** Namespace for all Analytics event types
 * @public
 */
export declare namespace FoundationAnalyticsEvent {
    /**
     * Represents an event that is triggered when a control is clicked.
     * @public
     */
    interface ControlClicked {
        name: string;
    }
    /**
     * Represents an event that is triggered when a control is clicked.
     * @public
     */
    interface RouteChanged {
        path: string;
        referrer?: string;
    }
}
/**
 * Represents an analytics tracking system.
 * @public
 */
export interface FoundationAnalytics {
    /**
     * Tracks a "control clicked" event with the specified payload.
     * @public
     * @param eventName - The name of the event to track.
     * @param payload - The payload data for the event.
     */
    trackEvent(eventName: FoundationAnalyticsEventType.controlClicked, payload: FoundationAnalyticsEvent.ControlClicked): void;
    /**
     * Tracks a "route changed" event with the specified payload.
     * @public
     * @param eventName - The name of the event to track.
     * @param payload - The payload data for the event.
     */
    trackEvent(eventName: FoundationAnalyticsEventType.routeChanged, payload: FoundationAnalyticsEvent.RouteChanged): void;
    /**
     * Tracks an event with the specified name and payload.
     * @public
     * @param eventName - The name of the event to track.
     * @param payload - The payload data for the event.
     */
    trackEvent(eventName: FoundationAnalyticsEventType, payload: {}): void;
}
/**
 * The default implementation for the FoundationAnalytics interface.
 * @public
 */
export declare class DefaultFoundationAnalytics implements FoundationAnalytics {
    private analytics;
    /**
     * TODO: Remove the static config.
     * We could register a config with the container at startup and have it injected here or remove the constructor and
     * define a method call.
     */
    constructor(config: {
        app: 'foundation-app';
        debug: true;
    });
    /**
     * Set typed overloads
     */
    trackEvent(eventName: FoundationAnalyticsEventType.controlClicked, payload: FoundationAnalyticsEvent.ControlClicked): void;
    trackEvent(eventName: FoundationAnalyticsEventType.routeChanged, payload: FoundationAnalyticsEvent.RouteChanged): void;
}
/**
 * The DI token for the FouundationAnalytics interface.
 * @internal
 */
export declare const FoundationAnalytics: import("@microsoft/fast-foundation").InterfaceSymbol<FoundationAnalytics>;
/**
 * Manual usage example:
 *
 * FoundationAnalyticsService.trackEvent(FoundationAnalyticsEventType.routeChanged, {
 *     path: "settings",
 * });
 *
 * See login mf.
 */
//# sourceMappingURL=analytics.d.ts.map