import { User } from '@genesislcap/foundation-user';
import { Observable } from 'rxjs';
import { Connect } from '../connect';
import { MessageBuilder } from '../connect/message';
import { CredentialManager } from '../credentialManager';
import { Session } from '../session';
import { AuthInfo, LoginResult, LogoutResult } from './auth.types';
/**
 * Auth is the main interface for authenticating with a Genesis Server endpoint.
 * @public
 */
export interface Auth {
    /**
     * Indicates whether Auth is working or is idle.
     * @public
     */
    isWorking: boolean;
    /** This property is intended to be used internally by the authentication system.
     * @internal
     * @remarks
     * TODO: For now, we are keeping `loggedUserResult` until all app and mf usages are reviewed.
     */
    readonly loggedUserResult: LoginResult;
    /**
     * A boolean value indicating whether or not the user is currently logged in.
     * @public
     */
    readonly isLoggedIn: boolean;
    /**
     * An Observable that emits a boolean value indicating whether or not the user is currently logged in.
     * @public
     */
    readonly isLoggedIn$?: Observable<boolean>;
    /**
     * The current user
     * @internal
     */
    readonly currentUser: User;
    /**
     * Logs in the user with the specified credentials.
     * @public
     * @param credentials - The user's login credentials.
     * @returns A Promise that resolves to the result of the login attempt.
     * @see AuthInfo
     */
    login(credentials: AuthInfo): Promise<LoginResult>;
    /**
     * Logs out the current user, if any.
     * @public
     * @returns A Promise that resolves to the result of the logout attempt.
     * @see LogoutResult
     */
    logout(): Promise<LogoutResult>;
    /**
     * Attempts to re-authenticate the user using the existing session data.
     * @public
     * @returns A Promise that resolves to `true` if the user was successfully re-authenticated,
     * or `false` if re-authentication failed.
     */
    reAuthFromSession(): Promise<boolean>;
}
/**
 * The default implementation for the Auth interface.
 * @public
 */
export declare class DefaultAuth implements Auth {
    private session;
    private connect;
    private messageBuilder;
    private credentialManager;
    currentUser: User;
    isWorking: boolean;
    loggedUserResult: LoginResult;
    private loggedUserResultChanged;
    private isLoggedInSubject;
    get isLoggedIn$(): Observable<boolean>;
    get isLoggedIn(): boolean;
    constructor(session: Session, connect: Connect, messageBuilder: MessageBuilder, credentialManager: CredentialManager, currentUser: User);
    login(credentials: AuthInfo): Promise<LoginResult>;
    logout(): Promise<LogoutResult>;
    reAuthFromSession(): Promise<boolean>;
    /**
     * Ensures that the connection is active.
     * @internal
     */
    private ensureConnection;
    /**
     * Gets the logged user from a Message result.
     * @param result - The Message result from auth request.
     * @returns The logged user info or null if the result is invalid.
     */
    private getLoggedUserFromMessage;
    /**
     * Applies the authentication results to the Auth instance.
     * @internal
     */
    private applyAuthResults;
    /**
     * Removes the authentication results from the Auth instance.
     * @internal
     */
    private removeAuthResults;
}
/**
 * The DI token for the Auth interface.
 * @internal
 */
export declare const Auth: import("@microsoft/fast-foundation").InterfaceSymbol<Auth>;
//# sourceMappingURL=auth.d.ts.map