import type { Message } from '../connect/message';
/**
 * The available ypes of authentication.
 * @public
 */
export declare enum AuthType {
    BASIC = "BASIC",
    SSO = "SSO",
    JWT = "JWT",
    REFRESH = "REFRESH"
}
/**
 * Represents basic authentication information.
 * @public
 */
export type BasicAuthInfo = {
    type: AuthType & AuthType.BASIC;
    username: string;
    password: string;
    mfaToken?: string;
};
/**
 * Represents SSO authentication information.
 * @public
 */
export type SsoAuthInfo = {
    type: AuthType & AuthType.SSO;
    ssoToken: string;
};
/**
 * Represents refresh token authentication information.
 * @public
 */
export type RefreshAuthInfo = {
    type: AuthType & AuthType.REFRESH;
    username: string;
    refreshToken: string;
    mfaToken: string;
};
/**
 * Represents JWT authentication information.
 * @public
 */
export type JwtAuthInfo = {
    type: AuthType & AuthType.JWT;
    jwt: string;
    domain: string;
};
/**
 * Union type of all authentication information types.
 * @public
 */
export type AuthInfo = BasicAuthInfo | SsoAuthInfo | JwtAuthInfo | RefreshAuthInfo;
/**
 * Represents the result of a user login operation.
 * @public
 */
export type LoginResult = {
    authToken: string;
    refreshToken: string;
    username: string;
    profiles: string[];
    permissions: string[];
    sessionId: string;
};
/**
 * Represents the result of a user logout operation.
 * @public
 */
export interface LogoutResult {
    /**
     * Indicates if the logout operation was successful.
     * @public
     */
    success: boolean;
    /**
     * Provides more details if the logout operation failed.
     * @public
     */
    error?: string;
}
/**
 * Represents an error that occurs during authentication.
 * @public
 */
export declare class AuthError extends Error {
    sent: Message;
    received?: Message;
    constructor(sent: Message, received?: Message, message?: string);
    isIgnorableError(): boolean;
    getMessageError(): string;
    getDetailsError(): string;
    hasPasswordExpiredError(): boolean;
    hasMFAError(): boolean;
    hasErrorCode(code: string): boolean;
    private errorDetailFormatter;
}
//# sourceMappingURL=auth.types.d.ts.map