import { Listener } from '@genesislcap/foundation-utils';
import type { MonitoredResourceChecks, MonitoredResourceDataLogon } from '../networkMonitor';
import type { Message, MessageDetails } from './message';
import type { SocketObservable } from './socket';
/**
 * @public
 */
export declare const CommitEventType = "connect-commit-event";
/**
 * @public
 */
export type CommitEventDetail = {
    eventName: string;
    message: Message;
};
/**
 * @public
 */
export type CommitEvent = CustomEvent<CommitEventDetail>;
/**
 * @public
 */
export type StreamEventFunctions = {
    onMessage?: Function;
    onError?: Function;
    resourceChecks?: MonitoredResourceChecks;
    resourceDataLogon?: MonitoredResourceDataLogon;
};
/**
 * @public
 */
export declare const StreamEventType = "connect-stream-event";
/**
 * @public
 */
export type StreamEventDetail = {
    functions: StreamEventFunctions;
    message: Message<MessageDetails.DataserverRequest>;
    resourceName: string;
    stream: SocketObservable<any>;
};
/**
 * @public
 */
export type StreamEvent = CustomEvent<StreamEventDetail>;
/**
 * @public
 */
export declare const StreamErrorEventType = "connect-stream-error-event";
/**
 * @public
 */
export type StreamErrorEventDetail = Pick<StreamEventDetail, 'resourceName' | 'stream'> & {
    error: any;
};
/**
 * @public
 */
export type StreamErrorEvent = CustomEvent<StreamErrorEventDetail>;
/**
 * @public
 */
export declare const StreamCompleteEventType = "connect-stream-complete-event";
/**
 * @public
 */
export type StreamCompleteEventDetail = Pick<StreamEventDetail, 'resourceName' | 'stream'>;
/**
 * @public
 */
export type StreamCompleteEvent = CustomEvent<StreamCompleteEventDetail>;
/**
 * @public
 */
export declare const MetadataEventType = "connect-metadata-event";
/**
 * @public
 */
export type MetadataEventDetail = {
    resourceName: string;
    message: Message<MessageDetails.MetaRequest>;
    error?: any;
};
/**
 * @public
 */
export type MetadataEvent = CustomEvent<MetadataEventDetail>;
/**
 * `ConnectEvents` provide hooks to respond to specific connect events.
 *
 * @public
 */
export interface ConnectEvents {
    /**
     * Add a stream listener.
     * @param listener - A listener callback function.
     * @param complete - A complete listener callback function.
     * @param error - A error listener callback function.
     */
    addStreamListener(listener: Listener<StreamEvent>, complete: Listener<StreamCompleteEvent>, error: Listener<StreamErrorEvent>): () => void;
    /**
     * Add a commit listener.
     * @param listener - A listener callback function.
     */
    addCommitListener(listener: Listener<CommitEvent>): () => void;
    /**
     * Add a metadata listener.
     * @param listener - A listener callback function.
     */
    addMetadataListener(listener: Listener<MetadataEvent>): () => void;
}
/**
 * `ConnectEventsEmitter` provide event emitting hooks to Connect.
 *
 * @public
 */
export interface ConnectEventsEmitter extends ConnectEvents {
    /**
     * Called by Connect.
     * @param resourceName - The name of the resource.
     * @param message - The message.
     * @param stream - The actual stream itself.
     * @param functions - Any functions provided.
     */
    onStreamEvent(resourceName: string, message: Message<MessageDetails.DataserverRequest>, stream: SocketObservable<any>, functions: StreamEventFunctions): void;
    /**
     * Called by Connect.
     * @param resourceName - The name of the resource.
     * @param stream - The actual stream itself.
     * @param error - The error that occurred.
     */
    onStreamErrorEvent(resourceName: string, stream: SocketObservable<any>, error: any): void;
    /**
     * Called by Connect.
     * @param resourceName - The name of the resource.
     * @param stream - The actual stream itself.
     */
    onStreamCompleteEvent(resourceName: string, stream: SocketObservable<any>): void;
    /**
     * Called by Connect.
     * @param eventName - The name of the event.
     * @param message - The message.
     */
    onCommitEvent(eventName: string, message: Message): void;
    /**
     * Called by Connect.
     * @param resourceName - The name of the resource.
     * @param message - The message.
     * @param error - Optional error.
     */
    onMetadataEvent(resourceName: string, message: Message<MessageDetails.MetaRequest>, error?: any): void;
}
/**
 * Default ConnectEvents implementation.
 * @public
 */
export declare class DefaultConnectEvents implements ConnectEventsEmitter {
    /** @internal */
    protected streamListeners: import("@genesislcap/foundation-utils").Observer<StreamEvent>;
    /** @internal */
    protected streamCompleteListeners: import("@genesislcap/foundation-utils").Observer<StreamCompleteEvent>;
    /** @internal */
    protected streamErrorListeners: import("@genesislcap/foundation-utils").Observer<StreamErrorEvent>;
    /** {@inheritDoc ConnectEvents.addStreamListener} */
    addStreamListener(listener: Listener<StreamEvent>, complete: Listener<StreamCompleteEvent>, error: Listener<StreamErrorEvent>): () => void;
    /** {@inheritDoc ConnectEventsEmitter.onStreamEvent} */
    onStreamEvent(resourceName: string, message: Message<MessageDetails.DataserverRequest>, stream: SocketObservable<any>, functions: StreamEventFunctions): void;
    /** {@inheritDoc ConnectEventsEmitter.onStreamErrorEvent} */
    onStreamErrorEvent(resourceName: string, stream: SocketObservable<any>, error: any): void;
    /** {@inheritDoc ConnectEventsEmitter.onStreamCompleteEvent} */
    onStreamCompleteEvent(resourceName: string, stream: SocketObservable<any>): void;
    /** @internal */
    protected commitListeners: import("@genesislcap/foundation-utils").Observer<CommitEvent>;
    /** {@inheritDoc ConnectEvents.addCommitListener} */
    addCommitListener(listener: Listener<CommitEvent>): () => void;
    /** {@inheritDoc ConnectEventsEmitter.onStreamEvent} */
    onCommitEvent(eventName: string, message: Message): void;
    /** @internal */
    protected metadataListeners: import("@genesislcap/foundation-utils").Observer<MetadataEvent>;
    /** {@inheritDoc ConnectEvents.addMetadataListener} */
    addMetadataListener(listener: Listener<MetadataEvent>): () => void;
    /** {@inheritDoc ConnectEventsEmitter.onMetadataEvent} */
    onMetadataEvent(resourceName: string, message: Message<MessageDetails.MetaRequest>, error?: any): void;
}
/**
 * ConnectEvents DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const ConnectEvents: import("@microsoft/fast-foundation").InterfaceSymbol<ConnectEvents>;
/**
 * ConnectEventsEmitter DI key.
 *
 * @privateRemarks
 * ConnectEventsEmitter is used by Connect.
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const ConnectEventsEmitter: import("@microsoft/fast-foundation").InterfaceSymbol<ConnectEventsEmitter>;
//# sourceMappingURL=connect.events.d.ts.map