import { User } from '@genesislcap/foundation-user';
import { JSONSerializer, UUID } from '@genesislcap/foundation-utils';
import { Observable, Subject, Subscription } from 'rxjs';
import { Session } from '../session';
import { ConnectConfig } from './connect.types';
import { Message, MessageBuilder } from './message';
import { SocketReconnectOptions } from './reconnectStrategy';
import { SocketStatus } from './socket.status';
import type { SocketConnectOptions } from './socket.types';
/**
 * Subject that wraps a WebSocketSubject and provides a Subject interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Subject.
 */
export declare class SocketSubject<T> extends Subject<T> {
    constructor();
}
/**
 * Observable that wraps a WebSocketSubject and provides an Observable interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Observable.
 */
export declare class SocketObservable<T> extends Observable<T> {
    constructor(...args: any[]);
}
/**
 * An alias to RxJS Subscription.
 * @public
 */
export { Subscription as SocketSubscription };
/**
 * The Socket message handler definition.
 * @public
 */
export declare class SocketMessageHandler {
    constructor(message: Message, onMessage: Function, onError: Function, isSubscription: boolean);
    /**
     * The message that was sent.
     * @public
     */
    get sentMessage(): Message;
    /**
     * The handler function for incoming messages.
     * @public
     */
    get onReceivedMessage(): Function;
    /**
     * The handler function for errors.
     * @public
     */
    get onReceivedError(): Function;
    /**
     * Whether the message handler is a subscription.
     * @public
     */
    get isSubscription(): boolean;
    /**
     * @internal
     */
    private _message;
    /**
     * @internal
     */
    private _onMessage;
    /**
     * @internal
     */
    private _onError;
    /**
     * @internal
     */
    private _isSubscription;
}
/**
 * Represents a WebSocket used to establish a communication channel between client and a Genesis server.
 * Provides methods for socket connection, sending and receiving WS messages.
 * @public
 */
export interface Socket extends Omit<SocketStatus, 'serialize' | 'onClose' | 'reconnectionLimitReached' | 'closedCode' | 'closedReason' | 'closedClean'> {
    connect(host: string, options?: SocketConnectOptions, reconnectOptions?: SocketReconnectOptions): Promise<boolean>;
    /**
     * Disconnects the socket and clears all subscriptions
     */
    reset(): void;
    send<T>(message: Message<T | any>, needsHandling?: boolean): Promise<Message | any>;
    sendForStream<T>(message: Message<T | any>, onMessage: Function, onError: Function, onComplete?: Function): Observable<Message>;
    /**
     * Sends a message and returns an observable that emits messages received from the server.
     * @alpha
     * @param message - The message to send.
     * @param onMessage - The function to call when a message is received.
     * @param onError - The function to call when an error is received.
     * @returns An observable that emits messages received from the server.
     */
    sendForStreamWithoutTeardown<T>(message: Message<T | any>, onMessage: Function, onError: Function): Observable<Message>;
    socketMessages(): SocketSubject<Message>;
    host: string;
}
/**
 * Default Socket implementation.
 * @public
 */
export declare class DefaultSocket implements Socket {
    private messageBuilder;
    private session;
    private serializer;
    private uuid;
    private status;
    /**
     * Temp putting the user in place to bridge the hasValidSession logic.
     */
    protected user: User;
    protected config: ConnectConfig;
    private readonly defaultInterval;
    private readonly secondsMultiplier;
    private websocket;
    private wsMessages;
    private wsMessageHandlers;
    get isConfigured(): boolean;
    get isConnecting(): boolean;
    get isConnected(): boolean;
    get isConnectedSubject(): import("rxjs").BehaviorSubject<boolean>;
    get isDisconnected(): boolean;
    get isDisconnectedByServer(): boolean;
    get isReconnecting(): boolean;
    /** {@inheritDoc SerializedSocketStatus.hasValidSession} */
    get hasValidSession(): boolean;
    set hasValidSession(value: boolean);
    get host(): string;
    private heartbeatIsEnabled;
    private heartbeatInterval;
    private heartbeatTicker;
    private url;
    private connectOptions;
    private reconnectOptions;
    private socketSubscription;
    constructor(messageBuilder: MessageBuilder, session: Session, serializer: JSONSerializer, uuid: UUID, status: SocketStatus, 
    /**
     * Temp putting the user in place to bridge the hasValidSession logic.
     */
    user: User, config: ConnectConfig);
    private get heartbeatLoggingIsEnabled();
    private get messageLoggingIsEnabled();
    connect(host?: string, connectOptions?: SocketConnectOptions, reconnectOptions?: SocketReconnectOptions): Promise<boolean>;
    private prepareHeartbeat;
    private reconnect;
    reset(): void;
    private cleanMessages;
    send<T>(message: Message<T | any>, needsHandling?: boolean): Promise<Message | any>;
    sendForStream<T>(message: Message<T | any>, onMessage: Function, onError: Function, onComplete?: Function): Observable<Message>;
    sendForStreamWithoutTeardown<T>(message: Message<T | any>, onMessage: Function, onError: Function): Observable<Message>;
    socketMessages: () => SocketSubject<Message>;
    private setMessageHandler;
    private sendDataLogoff;
    private removeMessageHandler;
    private getMessageHandler;
    private logMessageTypeFlow;
}
/**
 * The DI token for the Socket interface.
 * @internal
 */
export declare const Socket: import("@microsoft/fast-foundation").InterfaceSymbol<Socket>;
/**
 * Gets Socket from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getSocket } from '@genesislcap/foundation-comms';
 * ...
 * private socket = getSocket();
 * ```
 *
 * @public
 */
export declare function getSocket(): Socket;
//# sourceMappingURL=socket.d.ts.map