import { Auth } from '../auth';
import { Connect, Message, SocketObservable } from '../connect';
import type { FieldMetadata, FilteredDataServerResult, IndexDetail, MetadataDetail, RequestServerResult } from '../metadata/metadata.types';
import { GenesisResources } from '../resources';
import { DatasourceConfig } from './datasource.config';
import type { DatasourceOptions, DatasourceStatus, Dataview } from './datasource.types';
import { ResourceType } from './datasource.types';
/**
 * DatasourceMetadata interface.
 * @public
 */
export interface DatasourceMetadata {
    fetchMetadataRequired: boolean;
    readonly availableIndexes: IndexDetail[];
    readonly originalFieldDef: MetadataDetail[];
    readonly fieldMetadata: FieldMetadata[];
    readonly requestFields: MetadataDetail[];
    readonly customRequestFields: MetadataDetail[];
}
/**
 * Datasource interface.
 * @public
 * @remarks
 * The public interface available on the injected store fragment.
 */
export interface Datasource extends DatasourceMetadata {
    /**
     * @public
     */
    status: Partial<DatasourceStatus>;
    /**
     * Indicates whether the datasource has been initialized.
     * @public
     */
    initialized: boolean;
    /**
     * Indicates whether the datasource is read-only.
     * TODO: Should have is prefix
     * @public
     */
    readOnly: boolean;
    /**
     * The type of resource the datasource represents.
     * @public
     */
    resourceType: ResourceType;
    /**
     * A stream of data from the datasource.
     * @public
     */
    stream: SocketObservable<FilteredDataServerResult | RequestServerResult>;
    /**
     * Indicates whether the resourceName is valid or not.
     * @public
     * @deprecated - Please use {@link (GenesisResources:interface).isValidResource} instead.
     */
    validResourceName(resourceName: string): boolean;
    /**
     * Initializes the datasource with the given options and fetches metadata if specified.
     * @public
     * @param options - The options to initialize the datasource with.
     * @param fetchMeta - Optional flag to fetch metadata for the datasource. Defaults to true.
     * @param startStream - Optional flag to start the data stream. Defaults to true.
     * @returns A promise that resolves with a boolean indicating whether the initialization was successful.
     */
    init(options: DatasourceOptions, fetchMeta?: boolean, startStream?: boolean): Promise<boolean>;
    /**
     * Starts the data stream for the datasource.
     * @public
     */
    startStream(): Promise<void>;
    /**
     * Destroys the datasource.
     * @remarks This should be called to clean up any resources when the datasource is no longer needed.
     * @public
     */
    destroy(): void;
    /**
     * Fetches metadata for the specified resource name.
     * @public
     * @param resourceName - The name of the resource to fetch metadata for.
     * @returns A promise that resolves with an array of FieldMetadata.
     */
    getMetadata?(resourceName: string): Promise<Array<FieldMetadata>>;
    /**
     * Takes a snapshot of the data in the datasource.
     * @public
     * @param overrideParams - Optional parameters for taking the snapshot.
     */
    snapshot(overrideParams?: any): Promise<Message>;
    /**
     * Takes a filtered snapshot of the data in the datasource.
     * @public
     * @param rowId - Optional ID of the row to filter.
     */
    snapshotFiltered(rowId?: string): Promise<any[]>;
    /**
     * Updates the data in the datasource with the specified data.
     * @public
     * @param data - The data to update the datasource with.
     * @returns A promise that resolves when the data has been updated.
     */
    updateData?(data: Array<any>): Promise<void>;
    /**
     * Handles the specified data and returns it.
     * @public
     * @param data - The data to handle.
     * @returns The handled data.
     */
    dataHandler?(data: Array<any>): Array<any>;
    /**
     * Adds the specified dataview to the datasource.
     * @public
     * @param view - The dataview to add.
     */
    addView?(view: Dataview): void;
    /**
     * Removes the specified dataview from the datasource.
     * @public
     * @param view - The dataview to remove.
     */
    dropView?(view: Dataview): void;
    /**
     * Sets a filter on the datasource with the specified name, value, and type.
     * @public
     * @param name - The name of the filter to set.
     * @param value - The value of the filter to set.
     * @param type - The type of the filter to set.
     * @returns A promise that resolves when the filter has been set.
     */
    setFilter?(name: string, value: any, type?: string): Promise<void>;
    /**
     * Clears the filter with the specified name from the datasource.
     * @public
     * @param name - The name of the filter to clear.
     * @returns A promise that resolves when the filter has been cleared.
     */
    clearFilter?(name: string): Promise<void>;
    /**
     * Creates a new item in the datasource with the specified data.
     * @public
     * @param data - The data to create the item with.
     * @returns A promise that resolves with the ID of the created item.
     */
    createItem?(data: any): Promise<string>;
    /**
     * Deletes an item with the given ID from the datasource.
     * @public
     * @param itemId - The ID of the item to be deleted.
     * @returns A promise that resolves when the item has been deleted.
     */
    deleteItem?(itemId: string): Promise<void>;
    /**
     * Updates an item in the datasource with the given ID and data.
     * @public
     * @param itemId - The ID of the item to be updated.
     * @param data - The data to be updated for the item.
     * @returns A promise that resolves when the item has been updated.
     */
    updateItem?(itemId: string, data: any): Promise<void>;
}
/**
 * The default implementation for the Datasource interface.
 * @public
 */
export declare class DefaultDatasource implements Datasource {
    protected connect: Connect;
    protected auth: Auth;
    protected resources: GenesisResources;
    protected config: DatasourceConfig;
    /**
     * {@inheritDoc Datasource.status}
     */
    status: DatasourceStatus;
    /** {@inheritDoc DatasourceMetadata.fetchMetadataRequired} */
    fetchMetadataRequired: boolean;
    /** {@inheritDoc DatasourceMetadata.startStream} */
    startStreamRequired: boolean;
    /** {@inheritDoc DatasourceMetadata.originalFieldDef} */
    originalFieldDef: MetadataDetail[];
    /** {@inheritDoc DatasourceMetadata.fieldMetadata} */
    fieldMetadata: FieldMetadata[];
    /** {@inheritDoc DatasourceMetadata.requestFields} */
    requestFields: MetadataDetail[];
    /** {@inheritDoc DatasourceMetadata.customRequestFields} */
    customRequestFields: MetadataDetail[];
    /** {@inheritDoc DatasourceMetadata.availableIndexes} */
    availableIndexes: IndexDetail[];
    /** {@inheritDoc Datasource.initialized} */
    get initialized(): boolean;
    readOnly: boolean;
    protected options: DatasourceOptions;
    resourceType: ResourceType;
    stream: SocketObservable<FilteredDataServerResult | RequestServerResult>;
    constructor(connect: Connect, auth: Auth, resources: GenesisResources, config: DatasourceConfig);
    /**
     * @deprecated - Please use {@link (GenesisResources:interface).isValidResource} instead.
     */
    validResourceName(resourceName: string): boolean;
    init(options: DatasourceOptions, fetchMeta?: boolean, startStream?: boolean): Promise<boolean>;
    startStream(): Promise<void>;
    /** {@inheritDoc Datasource.destroy} */
    destroy(): void;
    get params(): any;
    snapshot(overrideParams?: any): Promise<Message>;
    snapshotFiltered(rowId?: string): Promise<any[]>;
    private createStream;
    private createDataserverStream;
    private createReqRepStream;
    protected fetchAndApplyMetadata(resourceName: string): Promise<void>;
    /**
     * @internal
     */
    protected reportStatus(detail?: Partial<DatasourceStatus>): void;
    /**
     * @internal
     */
    protected normaliseStatus(status: Partial<DatasourceStatus>): DatasourceStatus;
}
/**
 * The DI token for the Datasource interface.
 * @internal
 */
export declare const Datasource: import("@microsoft/fast-foundation").InterfaceSymbol<Datasource>;
/**
 * Get Datasource from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { FilteredDataServerResult, getDatasource, SocketObservable } from '@genesislcap/foundation-comms';
 * ...
 * private ds = getDatasource();
 * private dsSub: SocketObservable<FilteredDataServerResult>;
 * ...
 * const dsInit = await this.ds.init({
 *   criteria: '',
 *   isSnapshot: false,
 *   resourceName: 'ALL_POSITIONS',
 *   reverse: false,
 *   maxRows: 250,
 *   maxView: 1000,
 * });
 * if (!dsInit) {
 *   logger.debug(`Failed to initialise 'ALL_POSITIONS' datasource`);
 *   return;
 * }
 * this.dsSub = this.ds.stream.subscribe((result) => { ... });
 * ```
 *
 * @public
 */
export declare function getDatasource(): Datasource;
//# sourceMappingURL=datasource.d.ts.map