import type { FieldMetadata } from '../metadata/metadata.types';
/**
 * Types of Genesis resources.
 * @public
 */
export declare enum ResourceType {
    DATASERVER = "DATASERVER",
    EVENT_HANDLER = "EVENT_HANDLER",
    REQUEST_SERVER = "REQUEST_SERVER"
}
/**
 * Data Update definition with details of what data to add, drop, and/or modify.
 * @public
 */
export interface DataUpdate {
    add?: Array<any>;
    drop?: Array<any>;
    modify?: Array<any>;
}
/**
 * Dataview object
 * @public
 */
export interface Dataview {
    /**
     * Function to update the data in the dataview object with an object of optional add, drop, and modify arrays of data.
     * @param data - A DataUpdate object containing the data to add, drop, and/or modify.
     * @returns A promise that resolves when the update is complete.
     */
    update?: (data: DataUpdate) => Promise<void>;
    /**
     * An array of data objects representing the data in the dataview.
     */
    data: any[];
    /**
     * An array of objects representing the metadata of each column in the dataview.
     */
    columnsMetadata?: FieldMetadata[];
    /**
     * A boolean indicating whether the search functionality should be hidden in the UI.
     */
    hideSearch?: boolean;
}
/**
 * Options that can be passed when initializing a datasource.
 * @public
 */
export interface DatasourceOptions {
    /**
     * The name of the resource to use for the datasource.
     */
    resourceName?: string;
    /**
     * Whether the datasource is a snapshot or not.
     */
    isSnapshot?: boolean;
    /**
     * The criteria used to filter data in the datasource.
     * @remarks Works with both DATASERVER and REQUEST_SERVER resources.
     */
    criteria?: string;
    /**
     * The maximum number of rows to return from the datasource.
     * @remarks Works with both DATASERVER and REQUEST_SERVER resources.
     */
    maxRows?: number;
    /**
     * The current page/"view" number being displayed.
     * @remarks Works with both DATASERVER and REQUEST_SERVER resources.
     */
    viewNumber?: number;
    /**
     * The fields to include in the returned data.
     * @remarks Only works with DATASERVER resources.
     */
    fields?: string;
    /**
     * The maximum number of rows to track as part of a client "view"
     * @remarks Only works with DATASERVER resources.
     */
    maxView?: number;
    /**
     * Defines the behaviour of the client view when new rows are received in real time.
     * @remarks Only works with DATASERVER resources.
     */
    movingView?: boolean;
    /**
     * The field to use for sorting the data.
     * @remarks Only works with DATASERVER.
     */
    orderBy?: string;
    /**
     * Whether to return the data in reverse order or not.
     * @remarks Only works with DATASERVER resources.
     */
    reverse?: boolean;
    /**
     * Whether to disable polling when using request/reply
     * @remarks Only works with REQUEST_SERVER resources.
     */
    disablePolling?: boolean;
    /**
     * The polling interval to use when using request/reply.
     * @remarks Only works with REQUEST_SERVER resources.
     * @remarks in milliseconds
     */
    pollingInterval?: number;
    /**
     * The request to send to the server.
     * @remarks Only works with REQUEST_SERVER resources.
     */
    request?: any;
    /**
     * Whether the Datasource service will setup RQUEST object based on metadata.
     * @remarks Default is `true`. Only works with REQUEST_SERVER resources.
     */
    requestAutoSetup?: boolean;
}
/**
 * DatasourceStatus.
 * @public
 */
export type DatasourceStatus = {
    name: string;
    type: `${ResourceType}`;
    isInitialized: boolean;
    metadataRequired: boolean;
    hasMetadata: boolean;
};
/**
 * DatasourceStatusChanged.
 * @public
 */
export declare const DatasourceStatusChanged = "datasource-status-changed";
/**
 * DatasourceStatusChangedEvent.
 * @public
 */
export type DatasourceStatusChangedEvent = CustomEvent<DatasourceStatus>;
//# sourceMappingURL=datasource.types.d.ts.map