import type { DTOMapper } from '@genesislcap/foundation-utils';
import type { SocketSubscription } from '../connect/socket';
import type { FilteredDataServerResult, RowData } from '../metadata/metadata.types';
import { Datasource, DefaultDatasource } from './datasource';
import type { DatasourceOptions } from './datasource.types';
/**
 * This is a wip. We need to see if and where this fits into our strategic shared datasource solution.
 */
/**
 * EntityDatasourceInit interface.
 *
 * @alpha
 */
export interface EntityDatasourceInit<TDTO, TEntity> {
    options: Omit<DatasourceOptions, 'isSnapshot'>;
    mapper: DTOMapper<TDTO, TEntity>;
    fetchMeta?: boolean;
}
/**
 * EntityDatasource DI interface.
 *
 * @remarks
 * A simple streaming datasource cache used mainly by services that employ DTO mappers.
 *
 * @example
 * An example service using an underlying EntityDatasource:
 * ```ts
 * export class DefaultDomainService implements DomainService {
 *   constructor(
 *     @EntityDatasource protected datasource: EntityDatasource<DomainDTO, DomainEntity>,
 *     @DomainDTOMapper protected mapper: DomainDTOMapper,
 *     @optional(DomainServiceConfig) private config = defaultConfig
 *   ) {}
 *
 *   get initialized = () {
 *     return this.datasource.initialized;
 *   }
 *
 *   initialize = async (init) => {
 *     await this.datasource.initialize({
 *       mapper: this.mapper,
 *       options: {
 *         resourceName: this.config.resourceName,
 *         ...init,
 *       },
 *     });
 *     return this.initialized;
 *   };
 *
 *   list = () => this.datasource.cache;
 *
 *   get = async (id: string) => this.datasource.cache.find((entity) => entity.id === id);
 *
 *   action = async (id: string, message?: string): Promise<Message> => {
 *     return this.commitEvent('EVENT_DOMAIN_ACTION', { id, message });
 *   };
 *
 *   private async commitEvent(event: string, entity: Partial<DomainEntity>) {
 *     const msg = await this.connect.commitEvent(event, {
 *       DETAILS: this.mapper.toDTO(entity),
 *       IGNORE_WARNINGS: true,
 *       VALIDATE: false,
 *     });
 *     return messageOrThrow(msg);
 *   }
 * ```
 *
 * @privateRemarks
 * This is just a first pass that extends DefaultDatasource, but it may evolve into something standalone. It doesn't
 * cover snapshots by design right now, and there's some overlap with existing datasource logic atm. I haven't covered
 * all scenarios like data log just yet, as I'd like us to re-think some logic before doing that.
 *
 * Router converter functions would likely leverage services to resolve entities ahead of the enter phase.
 * ```ts
 * { path: '/{entity:DomainEntity}', element: MyRoute }
 * ...
 * this.routes.converter('DomainEntity', async (entityId: string) => {
 *   const entity = await this.domainService.get(entityId);
 *   return entity;
 * });
 * ```
 *
 * @alpha
 */
export interface EntityDatasource<TDTO, TEntity> extends Datasource {
    /**
     * @remarks
     * Initialize datasource.
     */
    initialize(init: EntityDatasourceInit<TDTO, TEntity>): Promise<boolean>;
    /**
     * @remarks
     * In-memory entity cache.
     *
     * @privateRemarks
     * Keeping it simple for now, but this could be an indexedDB table, perhaps with Dexie + liveQuery in-front.
     */
    readonly cache: TEntity[];
    /**
     * @remarks
     * Cache is empty / awaiting first result.
     */
    readonly isEmpty: boolean;
    /**
     * @remarks
     * Disconnect and reset cache.
     */
    disconnect(): void;
}
/**
 * Default EntityDatasource DI implementation.
 *
 * @alpha
 */
export declare class DefaultEntityDatasource<TDTO, TEntity> extends DefaultDatasource implements EntityDatasource<TDTO, TEntity> {
    protected mapper: DTOMapper<TDTO, TEntity>;
    protected sourceRef: string;
    protected subscription: SocketSubscription;
    protected subscriptionLoggedOff: boolean;
    protected entityCache: TEntity[];
    protected entityMap: Map<string, TEntity>;
    protected rowId: string;
    /** {@inheritDoc EntityDatasource.initialized} */
    protected _initialized: boolean;
    get initialized(): boolean;
    set initialized(value: boolean);
    /** {@inheritDoc EntityDatasource.initialize} */
    initialize: (init: EntityDatasourceInit<TDTO, TEntity>) => Promise<boolean>;
    /** {@inheritDoc EntityDatasource.cache} */
    get cache(): TEntity[];
    /** {@inheritDoc EntityDatasource.isEmpty} */
    get isEmpty(): boolean;
    /** {@inheritDoc EntityDatasource.disconnect} */
    disconnect: () => void;
    protected updateCache(result: FilteredDataServerResult): void;
    protected processRowDataUpdate(rowData: RowData[], inserting?: boolean): void;
}
/**
 * EntityDatasource DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const EntityDatasource: import("@microsoft/fast-foundation").InterfaceSymbol<EntityDatasource<any, any>>;
//# sourceMappingURL=entityDatasource.d.ts.map