import Bowser from 'bowser';
import type { StreamEventDetail } from '../connect/connect.events';
import type { Message } from '../connect/message';
import type { SocketObservable } from '../connect/socket';
import type { SerializedSocketStatus } from '../connect/socket.status';
/**
 * MonitoredResource.
 * @public
 */
export type MonitoredResource = SocketObservable<any>;
/**
 * MonitoredResourceChecksResult.
 * @public
 */
export interface MonitoredResourceChecksResult {
    isConnected: boolean;
    lastUpdated: number;
    lastMessage?: Message;
    timeout?: boolean;
}
/**
 * MonitoredResourceChecks.
 * @param sourceRef - The resource source ref.
 * @param resource - The resource reference.
 * @public
 */
export type MonitoredResourceChecks = (sourceRef: string, resource: MonitoredResource) => Promise<MonitoredResourceChecksResult>;
/**
 * MonitoredResourceMetadata.
 * @public
 */
export interface MonitoredResourceMetadata extends Partial<MonitoredResourceChecksResult>, Partial<StreamEventDetail> {
}
/**
 * MonitoredResourceDataLogon.
 * @param metadata - The resource metadata.
 * @public
 */
export type MonitoredResourceDataLogon = (metadata: MonitoredResourceMetadata) => void;
/**
 * ResourceHealth.
 * @public
 */
export type ResourceHealth = MonitoredResourceChecksResult & {
    name: string;
    sourceRef: string[];
    lastError?: any;
};
/**
 * SystemHealthStatus.
 * @public
 */
export type SystemHealthStatus = {
    /**
     * Is the browser or device working online.
     */
    isOnline: boolean;
    /**
     * Browser information.
     */
    browserInformation?: Bowser.Parser.ParsedResult;
    /**
     * Is the user authenticated.
     */
    isAuthenticated: boolean;
    /**
     * The username if authenticated.
     */
    username?: string;
    /**
     * Is everything online and connected.
     */
    isHealthy: boolean;
    /**
     * Socket status.
     */
    socket: SerializedSocketStatus;
    /**
     * Resource health information.
     */
    resources: Record<string, ResourceHealth>;
    /**
     * Resources are all connected flag.
     */
    resourcesAllConnected: boolean;
    /**
     * Last time the information was updated.
     */
    lastUpdated: number;
    /**
     * Next time the information will be updated.
     *
     * @privateRemarks
     * Overlays may use deltas for a visual countdown to next update.
     */
    nextUpdate: number;
    /**
     * First time the information was updated.
     *
     * @remarks
     * Can be used by applications to understand how-long it has been since the last refresh and to drive automations.
     */
    firstUpdated: number;
    /**
     * User requested run checks.
     */
    runAllChecks(): void;
    /**
     * User requested report stability to Genesis.
     */
    reportStabilityToGenesis(): Promise<boolean>;
};
/**
 * SystemHealthChanged.
 * @public
 */
export declare const SystemHealthChanged = "system-health-changed";
/**
 * SystemHealthChangedEvent.
 * @public
 */
export type SystemHealthChangedEvent = CustomEvent<SystemHealthStatus>;
/**
 * Genesis connect events that can be monitored to invoke resource checking.
 *
 * @remarks
 * 'steam' and 'commit' are monitored by default and will invoke resource checking. 'metadata' request failures are
 * monitored for the purpose of surfacing such errors to the console only. For now, 'metadata' errors are unrecoverable.
 *
 * @public
 */
export type GenesisConnectEvents = 'stream' | 'commit';
//# sourceMappingURL=types.d.ts.map