import { Connect, ResourcesMessage } from '../../connect';
import type { ResourceType } from '../../datasource';
import { GenesisResourcesConfig } from './config';
import type { ResourceItem } from './types';
/**
 * GenesisResources DI interface.
 * @remarks
 * A lazy shared Genesis Resources lookup system. N number of callers can request information in succession, but only
 * the first will invoke the server request, and each caller will receive their results when that returns.
 * @public
 */
export interface GenesisResources {
    /**
     * Checks if the resource exists
     * @param resourceName - The name of the resource to check.
     */
    isValidResource: (resourceName: string) => Promise<boolean>;
    /**
     * Gets the type of resource
     * @param resourceName - The name of the resource to get the type for.
     */
    getResourceTypeFor: (resourceName: string) => Promise<ResourceType>;
    /**
     * Destroy and garbage collect
     */
    destroy: () => void;
}
/**
 * Default GenesisResources implementation.
 * @public
 */
export declare class DefaultGenesisResources implements GenesisResources {
    protected connect: Connect;
    protected config: GenesisResourcesConfig;
    protected resources: ResourceItem[];
    protected resourcesPromise: Promise<ResourcesMessage>;
    constructor(connect: Connect, config: GenesisResourcesConfig);
    /** {@inheritDoc GenesisResources.isValidResource} */
    isValidResource(resourceName: string): Promise<boolean>;
    /** {@inheritDoc GenesisResources.getResourceTypeFor} */
    getResourceTypeFor(resourceName: string): Promise<ResourceType>;
    /** {@inheritDoc GenesisResources.destroy} */
    destroy(): void;
    protected getResources(): Promise<void>;
    protected postLoadedMessage(): void;
}
/**
 * GenesisResources DI key.
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export declare const GenesisResources: import("@microsoft/fast-foundation").InterfaceSymbol<GenesisResources>;
/**
 * Gets GenesisResources from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getGenesisResources } from '@genesislcap/foundation-comms';
 * ...
 * const genesisResources = getGenesisResources();
 * ```
 * @public
 */
export declare function getGenesisResources(): GenesisResources;
//# sourceMappingURL=genesis.d.ts.map