import type { Message } from '../connect';
/**
 * TODO: Review overlaps between user domain and general storage.
 * I think we should hand off to the credentialManager for some of these things.
 */
/**
 * The available browser storage types.
 * @public
 */
export type StorageType = 'local' | 'session';
/**
 * The keys used to store user information in the session storage.
 * @internal
 */
type UserInfoKeys = 'authToken' | 'ssoToken' | 'refreshToken' | 'sessionId' | 'returnUrl' | 'hostUrl' | 'gridOptions';
/**
 * Represents a session storage and its capabilities.
 * Provides methods to set and get items from the specified storage.
 * @public
 */
export interface Session {
    /**
     * The return URL for the session.
     * @public
     */
    returnUrl: string;
    /**
     * Sets the storage key prefix for the session.
     * @public
     * @param prefix - The prefix to set.
     */
    setStorageKeyPrefix(prefix: string): void;
    /**
     * Sets the authentication result for the session.
     * @public
     * @param info - The authentication result to set.
     */
    setAuthResult(info: Message): void;
    /**
     * Deletes the user from the session storage.
     * @public
     */
    deleteUserFromSessionStorage(): void;
    /**
     * Captures the return URL for the session.
     * @public
     */
    captureReturnUrl(): void;
    /**
     * Sets an item in the specified storage with the specified key and value.
     * @public
     * @param key - The key to set.
     * @param value - The value to set.
     * @param storage - The storage type to use.
     */
    setItem(key: string | UserInfoKeys, value: string, storage: StorageType): void;
    /**
     * Returns the value of the specified item in the specified storage.
     * @public
     * @param  key - The key to get.
     * @param storage - The storage type to use.
     * @returns - The value of the specified item.
     */
    getItem(key: string | UserInfoKeys, storage: StorageType): string;
    /**
     * Removes the item with the specified key from the specified storage.
     * @public
     * @param key - The key of the item to remove.
     * @param storage - The storage type to use.
     */
    removeItem(key: string | UserInfoKeys, storage: StorageType): void;
    /**
     * Sets an item in the local storage with the specified key and value.
     * @public
     * @param key - The key to set.
     * @param value - The value to set.
     */
    setLocalStorageItem(key: string | UserInfoKeys, value: string): void;
    /**
     * Sets an item in the session storage with the specified key and value.
     * @public
     * @param key - The key to set.
     * @param value - The value to set.
     */
    setSessionStorageItem(key: string | UserInfoKeys, value: string): void;
    /**
     * Returns the value of the item with the specified key from the local storage.
     * @public
     * @param key - The key of the item to get.
     * @returns - The value of the item with the specified key.
     */
    getLocalStorageItem(key: string | UserInfoKeys): string;
    /**
     * Returns the value of the item with the specified key from the session storage.
     * @public
     * @param key - The key of the item to get.
     * @returns - The value of the item with the specified key.
     */
    getSessionStorageItem(key: string | UserInfoKeys): string;
    /**
     * Removes the item with the specified key from the local storage.
     * @public
     * @param key - The key of the item to remove.
     */
    removeLocalStorageItem(key: string | UserInfoKeys): void;
    /**
     * Removes the item with the specified key from the session storage.
     * @public
     * @param key - The key of the item to remove.
     */
    removeSessionStorageItem(key: string | UserInfoKeys): void;
}
/**
 * The default implementation for the Session interface.
 * @public
 */
export declare class DefaultSession implements Session {
    /**
     * The prefix to use for all storage keys.
     * @internal
     */
    private storageKeyPrefix;
    returnUrl: string;
    deleteUserFromSessionStorage(): void;
    setStorageKeyPrefix(prefix: string): void;
    setAuthResult(result: Message): void;
    captureReturnUrl(): void;
    setItem(key: string | UserInfoKeys, value: string, storage: StorageType): void;
    getItem(key: string | UserInfoKeys, storage: StorageType): string;
    removeItem(key: string | UserInfoKeys, storage: StorageType): void;
    setLocalStorageItem(key: string | UserInfoKeys, value: string): void;
    setSessionStorageItem(key: string | UserInfoKeys, value: string): void;
    getLocalStorageItem(key: string | UserInfoKeys): string;
    getSessionStorageItem(key: string | UserInfoKeys): string;
    removeLocalStorageItem(key: string | UserInfoKeys): void;
    removeSessionStorageItem(key: string | UserInfoKeys): void;
}
/**
 * The DI token for the Session interface.
 * @internal
 */
export declare const Session: import("@microsoft/fast-foundation").InterfaceSymbol<Session>;
export {};
//# sourceMappingURL=session.d.ts.map