import { User } from '@genesislcap/foundation-user';
import { Connect } from '../../connect';
import { KVStorageConfig } from './kv-config.service';
import { KeyValue, KeyValueTransfer, KVValue, KVEventFailureResponse } from './kv.types';
/**
 * Represents a KV (key-value) data storage and its capabilities.
 * Provides methods to set and get items from the KV data storage.
 *
 * @remarks
 * Provides a key value storage api that abstracts the underlying storage technology.
 *
 * @privateRemarks
 * This entire package will likely evolve into a standalone repo; Storage PBC.
 * Need to create a design document covering how account and namespace ids may come into play.
 *
 * @alpha
 */
export interface KVStorage {
    delete(keys: string[]): Promise<true | KVEventFailureResponse>;
    get<T = KVValue>(key?: string): Promise<KeyValueTransfer<T>[]>;
    list(): Promise<string[]>;
    put(kvs: KeyValue<KVValue>[]): Promise<true | KVEventFailureResponse>;
}
/**
 * The default implementation for the KVStorage interface.
 * @alpha
 */
export declare class DefaultKVStorage implements KVStorage {
    private connect;
    protected user: User;
    private kvStorageConfig;
    constructor(connect: Connect, user: User, kvStorageConfig?: KVStorageConfig);
    /**
     * Determine whether KV Storage is available for use.
     * @internal
     */
    private checkAvailability;
    /**
     * Deletes the specified key-value pairs from the KV storage.
     * @alpha
     * @param kvs - The key names to delete.
     * @returns A promise that resolves to true if the operation was successful, or an object containing error and warning information if the operation failed.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.delete(['myKey']);
     *  logger.debug(result); // true or { ERROR: ..., WARNING: ... }
     *  ...
     * ```
     */
    delete(keys: string[]): Promise<true | KVEventFailureResponse>;
    /**
     * Gets the specified key-value pairs from the KV storage.
     * @remarks If no key is specified, all key-value pairs for the current user are returned.
     * @alpha
     * @param kv - The single key-value pair to get.
     * @returns A promise that resolves to an array of key-value pairs.
     *
     * @example
     * ```ts
     * ...
     *  const result = await this.kvStorage.get({ key: 'myKey' });
     *  logger.debug(result); // [{ key: 'myKey', value: 'myValue' }]
     *
     *  const result2 = await this.kvStorage.get();
     *  logger.debug(result); // [{ key: 'myKey', value: 'myValue' }, { key: 'myOtherKey', value: 'myOtherValue' }]
     *  ...
     * ```
     */
    get<T = KVValue>(key?: string): Promise<KeyValueTransfer<T>[]>;
    /**
     * Lists all keys for the user.
     * @alpha
     * @returns A promise that resolves to all keys for the user.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.list();
     *  logger.debug(result); // ['myKey', 'myOtherKey']
     *  ...
     * ```
     */
    list(): Promise<string[]>;
    /**
     * Inserts or updates the specified key-value pairs in the KV storage.
     * @alpha
     * @param kvs - The key-value pairs to insert or update.
     * @returns A promise that resolves to true if the operation was successful, or an object containing error and warning information if the operation failed.
     * @example
     * ```ts
     *  ...
     *  const result = await this.kvStorage.put([{ key: 'myKey', value: 'myValue' }]);
     *  logger.debug(result); // true or { ERROR: ..., WARNING: ... }
     *  ...
     * ```
     */
    put(kvs: KeyValue<KVValue>[]): Promise<true | KVEventFailureResponse>;
}
/**
 * The DI token for the KVStorage interface.
 * @internal
 */
export declare const KVStorage: import("@microsoft/fast-foundation").InterfaceSymbol<KVStorage>;
/**
 * Gets KVStorage from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getKVStorage } from '@genesislcap/foundation-comms';
 * ...
 * private kvStorage = getKVStorage();
 * ```
 *
 * @public
 */
export declare function getKVStorage(): KVStorage;
//# sourceMappingURL=kv.service.d.ts.map