import { __awaiter, __decorate, __param } from "tslib";
import { User } from '@genesislcap/foundation-user';
import { GENESIS_SOCKET_URL, JSONSerializer, UUID } from '@genesislcap/foundation-utils';
import { DOM, observable } from '@microsoft/fast-element';
import { DI } from '@microsoft/fast-foundation';
import { Observable, of, Subject, Subscription, takeUntil, throwError } from 'rxjs';
import { filter, mergeMap, switchMap, takeWhile } from 'rxjs/operators';
import { WebSocketSubject } from 'rxjs/webSocket';
import { Session } from '../session';
import { logger } from '../utils';
import { ConnectConfig } from './connect.types';
import { EventMessageType, MessageBuilder, MessageType, } from './message';
import { exponentialScheduler, linearScheduler, MAX_RECONNECT_ATTEMPTS, retryInterval, SocketReconnectStrategy, } from './reconnectStrategy';
import { SocketStatus } from './socket.status';
import { Ticker } from './ticker';
/**
 * Subject that wraps a WebSocketSubject and provides a Subject interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Subject.
 */
export class SocketSubject extends Subject {
    constructor() {
        super();
    }
}
/**
 * Observable that wraps a WebSocketSubject and provides an Observable interface for sending and receiving messages.
 * @public
 * @remarks An alias to RxJS Observable.
 */
export class SocketObservable extends Observable {
    constructor(...args) {
        super(...args);
    }
}
/**
 * An alias to RxJS Subscription.
 * @public
 */
export { Subscription as SocketSubscription };
/**
 * The Socket message handler definition.
 * @public
 */
export class SocketMessageHandler {
    constructor(message, onMessage, onError, isSubscription) {
        this._message = message;
        this._onMessage = onMessage;
        this._onError = onError;
        this._isSubscription = isSubscription;
    }
    /**
     * The message that was sent.
     * @public
     */
    get sentMessage() {
        return this._message;
    }
    /**
     * The handler function for incoming messages.
     * @public
     */
    get onReceivedMessage() {
        return this._onMessage;
    }
    /**
     * The handler function for errors.
     * @public
     */
    get onReceivedError() {
        return this._onError;
    }
    /**
     * Whether the message handler is a subscription.
     * @public
     */
    get isSubscription() {
        return this._isSubscription;
    }
}
/**
 * Default Socket implementation.
 * @public
 */
let DefaultSocket = class DefaultSocket {
    get isConfigured() {
        return this.status.isConfigured;
    }
    get isConnecting() {
        return this.status.isConnecting;
    }
    get isConnected() {
        return this.status.isConnected;
    }
    get isConnectedSubject() {
        return this.status.isConnectedSubject;
    }
    get isDisconnected() {
        return this.status.isDisconnected;
    }
    get isDisconnectedByServer() {
        return this.status.isDisconnectedByServer;
    }
    get isReconnecting() {
        return this.status.isReconnecting;
    }
    /** {@inheritDoc SerializedSocketStatus.hasValidSession} */
    get hasValidSession() {
        logger.deprecated('get hasValidSession', 'Use {@link @genesislcap/foundation-user#User.isAuthenticated} directly instead.', '15.0.0');
        return this.user.isAuthenticated;
    }
    set hasValidSession(value) {
        logger.deprecated('set hasValidSession', 'Please refrain from calling this API as this state is managed internally.', '15.0.0');
    }
    get host() {
        var _a;
        return (_a = this.url) !== null && _a !== void 0 ? _a : GENESIS_SOCKET_URL;
    }
    constructor(messageBuilder, session, serializer, uuid, status, 
    /**
     * Temp putting the user in place to bridge the hasValidSession logic.
     */
    user, config) {
        this.messageBuilder = messageBuilder;
        this.session = session;
        this.serializer = serializer;
        this.uuid = uuid;
        this.status = status;
        this.user = user;
        this.config = config;
        this.defaultInterval = 10000;
        this.secondsMultiplier = 1000;
        this.heartbeatIsEnabled = true;
        this.heartbeatInterval = this.defaultInterval;
        this.socketMessages = () => this.wsMessages;
    }
    get heartbeatLoggingIsEnabled() {
        var _a, _b;
        return (_b = (_a = this.connectOptions) === null || _a === void 0 ? void 0 : _a.enableHeartbeatLogging) !== null && _b !== void 0 ? _b : false;
    }
    get messageLoggingIsEnabled() {
        var _a, _b;
        return (_b = (_a = this.connectOptions) === null || _a === void 0 ? void 0 : _a.enableMessageLogging) !== null && _b !== void 0 ? _b : false;
    }
    connect(host = GENESIS_SOCKET_URL, connectOptions, reconnectOptions) {
        var _a;
        this.url = host;
        this.connectOptions = connectOptions !== null && connectOptions !== void 0 ? connectOptions : this.config.connect;
        this.reconnectOptions = reconnectOptions !== null && reconnectOptions !== void 0 ? reconnectOptions : this.config.reconnect;
        if (!((_a = this.config.isWebSocketExpression) === null || _a === void 0 ? void 0 : _a.test(this.host))) {
            logger.error(`Specified host (${this.host}) is not valid WebSocket url. Connection will not proceed.`);
            logger.warn(`Make sure the host url starts with wss:// or ws://`);
            return;
        }
        return new Promise((resolve) => {
            var _a, _b, _c, _d;
            logger.info(`${this.isReconnecting ? 'Reconnecting' : 'Connecting'} to ${this.url}`);
            this.status.isConnecting = !this.isReconnecting;
            const cleanMessages = !this.isReconnecting || !((_a = this.reconnectOptions) === null || _a === void 0 ? void 0 : _a.reconnectStreams);
            if (cleanMessages) {
                this.cleanMessages();
            }
            (_b = this.socketSubscription) === null || _b === void 0 ? void 0 : _b.unsubscribe();
            this.websocket = new WebSocketSubject({
                url: this.url,
                openObserver: {
                    next: () => {
                        this.session.setSessionStorageItem('hostUrl', this.url);
                        this.status.isConnected = true;
                        this.prepareHeartbeat();
                        logger.info(`WebSocket connection is now open`);
                        resolve(this.isConnected);
                    },
                },
                closeObserver: {
                    next: (closeEvent) => {
                        this.status.onClose(closeEvent);
                        if (this.isDisconnectedByServer) {
                            logger.info(`WebSocket connection was closed due an error - Code: ${closeEvent.code} ${closeEvent.reason ? ' | Reason: ' + closeEvent.reason : ''}`);
                        }
                        resolve(this.isConnected);
                    },
                },
                serializer: (object) => this.serializer.serialize(object),
                deserializer: (event) => this.serializer.deserialize(event),
            });
            const onNextMessage = (receivedMessage) => __awaiter(this, void 0, void 0, function* () {
                var _e;
                const responseMessageReceivedLog = `Response Message received for SourceRef: ${receivedMessage.SOURCE_REF} | MessageType: ${receivedMessage.MESSAGE_TYPE}`;
                this.logMessageTypeFlow(receivedMessage.MESSAGE_TYPE, logger.info, responseMessageReceivedLog);
                const { SOURCE_REF, MESSAGE_TYPE } = receivedMessage;
                const messageHandler = this.getMessageHandler(SOURCE_REF, MESSAGE_TYPE);
                if (!messageHandler) {
                    if (receivedMessage.MESSAGE_TYPE === MessageType.MSG_NACK && receivedMessage.ERROR) {
                        receivedMessage.ERROR.forEach((error) => {
                            logger.error(`MessageType: ${receivedMessage.MESSAGE_TYPE} | Error Details:`, error);
                        });
                    }
                    return;
                }
                let nacked = false;
                switch (receivedMessage.MESSAGE_TYPE) {
                    case EventMessageType.EVENT_LOGIN_AUTH_ACK: {
                        const systemDetails = receivedMessage.DETAILS;
                        // Reconfigures the heartbeat with system details from the login response
                        if (systemDetails.HEARTBEAT_INTERVAL_SECS &&
                            ((_e = this.connectOptions) === null || _e === void 0 ? void 0 : _e.heartbeatInterval) === undefined) {
                            this.heartbeatInterval =
                                systemDetails.HEARTBEAT_INTERVAL_SECS * this.secondsMultiplier;
                            this.prepareHeartbeat();
                        }
                        break;
                    }
                    case MessageType.LOGOUT_ACK: {
                        messageHandler.onReceivedMessage.call(this, receivedMessage);
                        this.cleanMessages();
                        return;
                    }
                    case MessageType.LOGON_ACK:
                        return;
                    case MessageType.LOGOFF_ACK: {
                        /**
                         * Needs review as I don't quite get the original logic, ie. LOGGED_OUT + hasValidSession.
                         * Perhaps these things should not be the concern of this actor. Temp putting the user in place instead.
                         */
                        if (receivedMessage.LOGGED_OUT && this.user.isAuthenticated) {
                            const refreshTokenMessage = this.messageBuilder.createRefreshTokenMessage();
                            const refreshTokenResult = yield this.send(refreshTokenMessage);
                            if ((refreshTokenResult === null || refreshTokenResult === void 0 ? void 0 : refreshTokenResult.MESSAGE_TYPE) === EventMessageType.EVENT_LOGIN_AUTH_ACK) {
                                this.session.setAuthResult(refreshTokenResult);
                                this.wsMessageHandlers.forEach((mh) => {
                                    if (mh.isSubscription) {
                                        mh.sentMessage.SESSION_AUTH_TOKEN = refreshTokenResult.SESSION_AUTH_TOKEN;
                                        this.websocket.next(mh.sentMessage);
                                    }
                                });
                            }
                        }
                        break;
                    }
                    case MessageType.QUERY_UPDATE: {
                        if (!messageHandler.isSubscription) {
                            this.sendDataLogoff(SOURCE_REF);
                        }
                        break;
                    }
                    case MessageType.MSG_NACK:
                    case MessageType.LOGON_NACK: {
                        nacked = true;
                        break;
                    }
                }
                if (nacked) {
                    messageHandler.onReceivedError.call(this, {
                        receivedMessage,
                        sentMessage: messageHandler.sentMessage,
                    });
                }
                else {
                    messageHandler.onReceivedMessage.call(this, receivedMessage);
                }
                this.wsMessages.next(receivedMessage);
                if (!messageHandler.isSubscription) {
                    this.removeMessageHandler(SOURCE_REF);
                }
            });
            const onError = (error) => {
                logger.debug('Error found: ', error);
                this.reconnect();
            };
            const onComplete = () => {
                var _a;
                if (this.isDisconnectedByServer && ((_a = this.connectOptions) === null || _a === void 0 ? void 0 : _a.alwaysOn)) {
                    this.reconnect();
                    return;
                }
                logger.info('Connection is now closed');
            };
            this.socketSubscription = this.websocket.subscribe((_d = (_c = this.connectOptions) === null || _c === void 0 ? void 0 : _c.connectObserver) !== null && _d !== void 0 ? _d : {
                next: onNextMessage,
                error: onError,
                complete: onComplete,
            });
            this.status.isConfigured = true;
        });
    }
    prepareHeartbeat() {
        var _a, _b, _c, _d;
        if (this.heartbeatTicker) {
            this.heartbeatTicker.stop();
        }
        const sendHeartbeat = () => __awaiter(this, void 0, void 0, function* () {
            var _e, _f, _g;
            if (!this.isConnected)
                return;
            const ping = this.messageBuilder.createHeartbeatPingMessage((_e = this.connectOptions) === null || _e === void 0 ? void 0 : _e.verboseHeartbeat);
            if ((_f = this.connectOptions) === null || _f === void 0 ? void 0 : _f.heartbeatPingCallback) {
                this.connectOptions.heartbeatPingCallback(ping);
            }
            const pong = yield this.send(ping);
            if ((_g = this.connectOptions) === null || _g === void 0 ? void 0 : _g.heartbeatPongCallback) {
                this.connectOptions.heartbeatPongCallback(pong);
            }
            this.heartbeatIsEnabled =
                pong.MESSAGE_TYPE === MessageType.HEARTBEAT_PONG && pong.SOURCE_REF === ping.SOURCE_REF;
            if (pong.IS_AUTHENTICATED !== undefined) {
                this.hasValidSession = pong.IS_AUTHENTICATED;
            }
        });
        if ((_b = (_a = this.connectOptions) === null || _a === void 0 ? void 0 : _a.enableHeartbeat) !== null && _b !== void 0 ? _b : this.heartbeatIsEnabled) {
            const intervalInMillisecocnds = (_d = (_c = this.connectOptions) === null || _c === void 0 ? void 0 : _c.heartbeatInterval) !== null && _d !== void 0 ? _d : this.heartbeatInterval;
            this.heartbeatTicker = new Ticker(intervalInMillisecocnds);
            this.heartbeatTicker.start(() => {
                if (this.websocket && this.isConnected) {
                    sendHeartbeat();
                    if (this.heartbeatLoggingIsEnabled) {
                        logger.debug(`Sending a heartbeat ping every ${intervalInMillisecocnds} ms`);
                    }
                }
            });
        }
    }
    reconnect() {
        var _a, _b, _c, _d, _e;
        const notAlwaysOnAndHeartbeating = !((_a = this.connectOptions) === null || _a === void 0 ? void 0 : _a.alwaysOn) && ((_b = this.connectOptions) === null || _b === void 0 ? void 0 : _b.enableHeartbeat) === false;
        if (this.isReconnecting || notAlwaysOnAndHeartbeating) {
            return;
        }
        this.status.isReconnecting = true;
        const scheduler = ((_c = this.reconnectOptions) === null || _c === void 0 ? void 0 : _c.reconnectStrategy) === SocketReconnectStrategy.Exponential
            ? exponentialScheduler
            : linearScheduler;
        const reconnectObservable = this.isConnectedSubject.asObservable().pipe(switchMap((isConnected) => !isConnected
            ? retryInterval(this.reconnectOptions, scheduler).pipe(mergeMap((value) => {
                var _a, _b, _c;
                let attemptsAmount;
                if ((_a = this.connectOptions) === null || _a === void 0 ? void 0 : _a.alwaysOn) {
                    attemptsAmount = value + 1;
                }
                else {
                    attemptsAmount =
                        (_c = (_b = this.reconnectOptions) === null || _b === void 0 ? void 0 : _b.reconnectAttempts) !== null && _c !== void 0 ? _c : MAX_RECONNECT_ATTEMPTS;
                }
                if (value < attemptsAmount) {
                    return of(value);
                }
                this.status.reconnectionLimitReached = true;
                return throwError(() => new Error(`Max reconnect attempts ${attemptsAmount} reached.`));
            }))
            : of(-1)), takeWhile(() => this.websocket && !this.isConnected));
        reconnectObservable.subscribe((_e = (_d = this.reconnectOptions) === null || _d === void 0 ? void 0 : _d.reconnectObserver) !== null && _e !== void 0 ? _e : {
            next: (value) => {
                logger.debug(`Attempt no. ${value + 1} to reconnect`);
                this.websocket.complete();
                this.connect(this.url, this.connectOptions, this.reconnectOptions);
            },
            error: (error) => {
                var _a;
                logger.debug(error);
                (_a = this.socketSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                this.status.isReconnecting = false;
            },
            complete: () => {
                logger.info(this.isConnected ? 'Reconnected' : 'Disconnected');
                this.status.isReconnecting = false;
                this.wsMessageHandlers.forEach((mh) => {
                    if (mh.isSubscription) {
                        this.websocket.next(mh.sentMessage);
                    }
                });
            },
        });
    }
    reset() {
        this.status.isConnected = false;
        this.status.isConfigured = false;
        this.websocket && this.websocket.complete();
        this.websocket = null;
        this.cleanMessages();
    }
    cleanMessages() {
        this.wsMessages && this.wsMessages.complete();
        this.wsMessages = new SocketSubject();
        this.wsMessageHandlers = new Map();
    }
    send(message, needsHandling = true) {
        if (!this.isConnected) {
            return Promise.reject(new Error('WebSocket connection is closed'));
        }
        const sendingMessasgeLog = `Sending Message for SourceRef: ${message.SOURCE_REF} | MessageType: ${message.MESSAGE_TYPE}`;
        this.logMessageTypeFlow(message.MESSAGE_TYPE, logger.info, sendingMessasgeLog);
        return new Promise((resolve, reject) => {
            const onMessage = (msg) => {
                resolve(msg);
            };
            const onError = (err) => {
                reject(err);
            };
            if (needsHandling) {
                this.setMessageHandler(message, onMessage, onError);
            }
            this.websocket.next(message);
        });
    }
    sendForStream(message, onMessage, onError, onComplete) {
        if (!this.isConnected) {
            return throwError(() => new Error('WebSocket connection is closed'));
        }
        const sendingMessageLog = `Sending Message (stream) for SourceRef: ${message.SOURCE_REF} | MessageType: ${message.MESSAGE_TYPE}`;
        this.logMessageTypeFlow(message.MESSAGE_TYPE, logger.info, sendingMessageLog);
        this.setMessageHandler(message, onMessage, onError, true);
        /**
         * We need to queue these outgoing messages to allow the consumers to subscribe in time. We should review the entire
         * setup with late subscribers in mind, as we can't say when they will actually subscribe. The impact is grids and
         * other components remaining stuck on Loading screens, as they miss the first `onMessage` reply.
         */
        DOM.queueUpdate(() => this.websocket.next(message));
        /**
         * @privateRemarks
         * Running the data logoff from here makes more sense given the network monitor actor is optionally added.
         */
        return new Observable((subscriber) => {
            const destroyNotifier = new Subject();
            this.wsMessages
                .pipe(filter((msg) => msg.SOURCE_REF === message.SOURCE_REF), takeUntil(destroyNotifier))
                .subscribe({
                next: (value) => {
                    subscriber.next(value);
                },
                error: (err) => {
                    subscriber.error(err);
                },
                complete: () => {
                    subscriber.complete();
                },
            });
            return () => {
                destroyNotifier.complete();
                this.sendDataLogoff(message.SOURCE_REF);
                this.removeMessageHandler(message.SOURCE_REF);
                onComplete && onComplete();
            };
        });
    }
    sendForStreamWithoutTeardown(message, onMessage, onError) {
        if (!this.isConnected) {
            return throwError(() => new Error('WebSocket connection is closed'));
        }
        const sendingMessageLog = `Sending Message (stream) for SourceRef: ${message.SOURCE_REF} | MessageType: ${message.MESSAGE_TYPE}`;
        this.logMessageTypeFlow(message.MESSAGE_TYPE, logger.info, sendingMessageLog);
        this.setMessageHandler(message, onMessage, onError, true);
        this.websocket.next(message);
        return this.wsMessages.pipe(filter((msg) => msg.SOURCE_REF === message.SOURCE_REF));
    }
    setMessageHandler(message, onMessage, onError, isSubscription = false) {
        const settingMessageHandlerLog = `Setting Message Handler for SourceRef: ${message.SOURCE_REF} | MessageType: ${message.MESSAGE_TYPE}`;
        this.logMessageTypeFlow(message.MESSAGE_TYPE, logger.debug, settingMessageHandlerLog);
        this.wsMessageHandlers.set(message.SOURCE_REF, new SocketMessageHandler(message, onMessage, onError, isSubscription));
    }
    sendDataLogoff(sourceRef) {
        this.send(this.messageBuilder.createDataLogoffMessage(sourceRef), false);
    }
    removeMessageHandler(sourceRef) {
        this.wsMessageHandlers.delete(sourceRef);
    }
    getMessageHandler(sourceRef, messageType) {
        const gettingMessageHandlerLog = `Getting Message Handler for SourceRef: ${sourceRef} | MessageType: ${messageType}`;
        this.logMessageTypeFlow(messageType, logger.debug, gettingMessageHandlerLog);
        return this.wsMessageHandlers.get(sourceRef);
    }
    logMessageTypeFlow(messageType, logFunc, logMsg) {
        switch (messageType) {
            case EventMessageType.DATA_LOGOFF:
                logFunc(logMsg);
                break;
            case MessageType.HEARTBEAT_PING:
            case MessageType.HEARTBEAT_PONG:
                if (this.heartbeatLoggingIsEnabled) {
                    logFunc(logMsg);
                }
                break;
            default:
                if (this.messageLoggingIsEnabled) {
                    logFunc(logMsg);
                }
                break;
        }
    }
};
__decorate([
    observable
], DefaultSocket.prototype, "websocket", void 0);
__decorate([
    observable
], DefaultSocket.prototype, "wsMessages", void 0);
__decorate([
    observable
], DefaultSocket.prototype, "wsMessageHandlers", void 0);
__decorate([
    observable
], DefaultSocket.prototype, "heartbeatIsEnabled", void 0);
DefaultSocket = __decorate([
    __param(0, MessageBuilder),
    __param(1, Session),
    __param(2, JSONSerializer),
    __param(3, UUID),
    __param(4, SocketStatus),
    __param(5, User),
    __param(6, ConnectConfig)
], DefaultSocket);
export { DefaultSocket };
/**
 * The DI token for the Socket interface.
 * @internal
 */
export const Socket = DI.createInterface((x) => x.singleton(DefaultSocket));
/**
 * Gets Socket from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getSocket } from '@genesislcap/foundation-comms';
 * ...
 * private socket = getSocket();
 * ```
 *
 * @public
 */
export function getSocket() {
    return DI.getOrCreateDOMContainer().get(Socket);
}
