import { __decorate } from "tslib";
import { observable } from '@microsoft/fast-element';
import { logger } from '../utils';
import { DatasourceStatusChanged, } from './datasource.types';
/**
 * Datasource event handler mixin for target classes that use or are interested in datasource status changes.
 *
 * @remarks
 * Classes that use the mixin are not required to own the datasource itself, there may be several interested parties.
 *
 * @example
 * ```ts
 * import { DatasourceEventHandler, DatasourceStatus } from '@genesislcap/foundation-comms';
 * ...
 * class MyComponent extends DatasourceEventHandler(FASTElement) {
 *     resourceName: string = 'some-resource';
 *     ...
 *     datasourceStatusChanged(prev: DatasourceStatus, next: DatasourceStatus) {
 *         // Respond to changes in the target datasource.
 *     }
 * }
 * ```
 *
 * @privateRemarks
 * Unfortunately our datasource ui components do not share a common baseclass. Given datasource ui components contain a
 * Datasource and currently await their `init`, usage is more for example at this point, ie. how could we allow interested
 * parties to monitor and respond to changes, given they will exist at different levels of DOM hierarchy etc. Datasource
 * itself is also transient DI injectable, so each interested party would not have access to the same instance, nor would
 * we necessarily want to expose that via the event to prevent them operating on it to cause unwanted side effects.
 *
 * @param Target - The class to mix the event handler with.
 *
 * @public
 */
export const DatasourceEventHandler = (Target) => {
    // @ts-ignore
    class Mixin extends Target {
        constructor() {
            super();
            if (!this.hasOwnProperty('onDatasourceStatusChanged')) {
                this.onDatasourceStatusChanged = (event) => {
                    var _a;
                    if (!this.resourceName) {
                        logger.warn(`Target classes must have a resourceName property to use the DatasourceEventHandler mixin.`);
                    }
                    const next = event.detail;
                    if (next.name === this.resourceName) {
                        const prev = (_a = this.datasourceStatus) !== null && _a !== void 0 ? _a : {
                            isInitialized: false,
                            hasMetadata: false,
                        };
                        const hasChanged = prev.isInitialized !== next.isInitialized ||
                            (next.metadataRequired && prev.hasMetadata !== next.hasMetadata);
                        if (hasChanged) {
                            this.datasourceStatus = next;
                        }
                    }
                };
            }
        }
        /**
         * @remarks
         * Override this method to respond to changes in the target datasource.
         */
        datasourceStatusChanged(prev, next) {
            logger.debug('Target classes should override the datasourceStatusChanged method of the DatasourceEventHandler mixin to respond to changes in the target datasource.');
        }
        connectedCallback() {
            super.connectedCallback();
            document.addEventListener(DatasourceStatusChanged, this.onDatasourceStatusChanged, {
                passive: true,
                capture: true,
            });
        }
        disconnectedCallback() {
            super.disconnectedCallback();
            document.removeEventListener(DatasourceStatusChanged, this.onDatasourceStatusChanged, {
                capture: true,
            });
        }
    }
    __decorate([
        observable
    ], Mixin.prototype, "datasourceStatus", void 0);
    return Mixin;
};
