import { __awaiter, __decorate, __param } from "tslib";
import { DI } from '@microsoft/fast-foundation';
import { Connect } from '../../connect';
import { logger } from '../../utils';
import { GenesisResourcesConfig, getGenesisResourcesChannel } from './config';
/**
 * Default GenesisResources implementation.
 * @public
 */
let DefaultGenesisResources = class DefaultGenesisResources {
    constructor(connect, config) {
        this.connect = connect;
        this.config = config;
    }
    /** {@inheritDoc GenesisResources.isValidResource} */
    isValidResource(resourceName) {
        return __awaiter(this, void 0, void 0, function* () {
            resourceName = resourceName === null || resourceName === void 0 ? void 0 : resourceName.trim();
            if (!resourceName) {
                return false;
            }
            yield this.getResources();
            const resourceItem = this.resources.find((item) => item.RESOURCE_NAME === resourceName);
            return !!resourceItem;
        });
    }
    /** {@inheritDoc GenesisResources.getResourceTypeFor} */
    getResourceTypeFor(resourceName) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getResources();
            const resourceItem = this.resources.find((item) => item.RESOURCE_NAME === resourceName);
            if (resourceItem) {
                return resourceItem.RESOURCE_TYPE;
            }
            const message = `${resourceName} resource does not exist.`;
            logger.error(message);
            if (this.config.throwWhenUnknown) {
                throw new Error(message);
            }
        });
    }
    /** {@inheritDoc GenesisResources.destroy} */
    destroy() {
        this.resources = undefined;
        this.resourcesPromise = undefined;
    }
    getResources() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.resourcesPromise) {
                /**
                 * Await a resources request
                 */
                this.resourcesPromise = this.connect.getAvailableResources();
                const data = yield this.resourcesPromise;
                this.resources = data === null || data === void 0 ? void 0 : data.RESOURCES;
                if (!((_a = this.resources) === null || _a === void 0 ? void 0 : _a.length)) {
                    const message = `Genesis resources unavailable.`;
                    logger.error(message);
                    if (this.config.throwWhenUnavailable) {
                        throw new Error(message);
                    }
                }
                this.postLoadedMessage();
                this.resourcesPromise = undefined;
            }
            else {
                /**
                 * Await the in-progress resources request
                 */
                yield this.resourcesPromise;
            }
        });
    }
    postLoadedMessage() {
        /**
         * Normally we'd inject channels into components and call their `channel.close()` api on `disconnectedCallback`.
         * However, in shared long-lived service instances like this, channels may lead to tests failing to exit as they are
         * not garbage collected by default. Creating and destroying the channel on on-demand poses less risk.
         */
        const channel = getGenesisResourcesChannel();
        channel.postMessage('resources-loaded', this.resources);
        channel.close();
    }
};
DefaultGenesisResources = __decorate([
    __param(0, Connect),
    __param(1, GenesisResourcesConfig)
], DefaultGenesisResources);
export { DefaultGenesisResources };
/**
 * GenesisResources DI key.
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export const GenesisResources = DI.createInterface((x) => x.singleton(DefaultGenesisResources));
/**
 * Gets GenesisResources from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getGenesisResources } from '@genesislcap/foundation-comms';
 * ...
 * const genesisResources = getGenesisResources();
 * ```
 * @public
 */
export function getGenesisResources() {
    return DI.getOrCreateDOMContainer().get(GenesisResources);
}
